@@ -12,9 +12,7 @@
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmarshal.h>
 #include <gtk/gtktypeutils.h>
-#if 0
 #include <gconf/gconf-client.h>
-#endif
 
 #include <gnome.h>
 
@@ -22,6 +20,8 @@
 #include <ggv-control.h>
 
 #include <gtkgs.h>
+#include <gsdefaults.h>
+#include <ggvutils.h>
 
 #include <bonobo.h>
 
@@ -37,9 +37,7 @@ struct _GgvPostscriptViewPrivate {
 	gdouble prev_x, prev_y;
 	gchar *tmp_name;
 
-#if 0
-	GConfClient	      *client;
-#endif
+	GConfClient	*conf_client;
 
 	BonoboPropertyBag     *property_bag;
 	BonoboPropertyControl *property_control;
@@ -661,57 +659,6 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 	}
 }
 	
-static GSList *
-split_string (const gchar *string, const gchar *delimiter)
-{
-	const gchar *ptr = string;
-	int pos = 0, escape = 0;
-	char buffer [BUFSIZ];
-	GSList *list = NULL;
-
-	g_return_val_if_fail (string != NULL, NULL);
-	g_return_val_if_fail (delimiter != NULL, NULL);
-
-	while (*ptr) {
-		char c = *ptr++;
-		const gchar *d;
-		int found = 0;
-
-		if (pos >= BUFSIZ) {
-			g_warning ("string too long, aborting");
-			return list;
-		}
-
-		if (escape) {
-			buffer [pos++] = c;
-			escape = 0;
-			continue;
-		}
-
-		if (c == '\\') {
-			escape = 1;
-			continue;
-		}
-
-		for (d = delimiter; *d; d++) {
-			if (c == *d) {
-				buffer [pos++] = 0;
-				list = g_slist_prepend (list, g_strdup (buffer));
-				pos = 0; found = 1;
-				break;
-			}
-		}
-
-		if (!found)
-			buffer [pos++] = c;
-	}
-
-	buffer [pos++] = 0;
-	list = g_slist_prepend (list, g_strdup (buffer));
-
-	return list;
-}
-
 static Bonobo_Unknown
 ggv_postscript_view_get_object(BonoboItemContainer *item_container,
 							   CORBA_char          *item_name,
@@ -729,7 +676,7 @@ ggv_postscript_view_get_object(BonoboItemContainer *item_container,
 	g_message ("ggv_postscript_view_get_object: %d - %s",
 			   only_if_exists, item_name);
 
-	params = split_string (item_name, "!");
+	params = ggv_split_string (item_name, "!");
 	for (c = params; c; c = c->next) {
 		gchar *name = c->data;
 
@@ -952,9 +899,7 @@ ggv_postscript_view_destroy(GtkObject *object)
 
 	ps_view = GGV_POSTSCRIPT_VIEW(object);
 
-#if 0
-	gtk_object_unref(GTK_OBJECT(ps_view->priv->client));
-#endif
+	gtk_object_unref(GTK_OBJECT(ps_view->priv->conf_client));
 
 	if(ps_view->priv->property_bag)
 		bonobo_object_unref(BONOBO_OBJECT(ps_view->priv->property_bag));
@@ -1120,13 +1065,11 @@ ggv_postscript_view_construct(GgvPostscriptView *ps_view,
 	g_return_val_if_fail(GTK_IS_GS(gs), NULL);
 	g_return_val_if_fail(!GTK_WIDGET_REALIZED(gs), NULL);
 
-#if 0
 	/* Make sure GConf is initialized */
 	if(!gconf_is_initialized())
 		gconf_init(0, NULL, NULL);
 	
-	ps_view->priv->client = gconf_client_get_default();
-#endif
+	ps_view->priv->conf_client = gconf_client_get_default();
 
 	ps_view->priv->gs = GTK_WIDGET(gs);
 	gtk_widget_set_events(GTK_WIDGET(gs), 
