@@ -12,6 +12,7 @@
 #include "prefs.h"
 #include "ggvwindow.h"
 #include "gsmessage.h"
+#include "gsdefaults.h"
 #include "callbacks.h"
 
 static void save_window_size(GtkWidget *window, gint *w, gint *h)
@@ -219,6 +220,7 @@ void button_press_callback(GtkWidget *widget, GdkEventButton *event,
                                          pan_cursor, GDK_CURRENT_TIME);
                         ggv->prev_x = wx;
                         ggv->prev_y = wy;
+                        gtk_gs_start_scroll(GTK_GS(ggv->gs));
                 }
                 break;
 	}
@@ -234,7 +236,7 @@ void button_press_callback(GtkWidget *widget, GdkEventButton *event,
 
                 cx = (gfloat)x/(gfloat)GTK_GS(ggv->gs)->width;
                 cy = (gfloat)y/(gfloat)GTK_GS(ggv->gs)->height;
-                zoom_to(ggv, ggv->zoom_magstep + 1);
+                zoom_to(ggv, ggv_compute_spec(GTK_GS(ggv->gs)->zoom_factor) + 1);
                 gtk_gs_set_center(GTK_GS(ggv->gs), cx, cy);
                 break;
         }
@@ -256,7 +258,7 @@ void button_press_callback(GtkWidget *widget, GdkEventButton *event,
 		if(event->state & GDK_CONTROL_MASK)
 			gtk_gs_scroll(GTK_GS(ggv->gs), -64, 0);
 		else if(event->state & GDK_SHIFT_MASK)
-			zoom_to(ggv, ggv->zoom_magstep+1);
+			zoom_to(ggv, ggv_compute_spec(GTK_GS(ggv->gs)->zoom_factor)+1);
 		else
 			gtk_gs_scroll(GTK_GS(ggv->gs), 0, -64);
 		break;
@@ -265,7 +267,7 @@ void button_press_callback(GtkWidget *widget, GdkEventButton *event,
 		if(event->state & GDK_CONTROL_MASK)
 			gtk_gs_scroll(GTK_GS(ggv->gs), 64, 0);
 		else if(event->state & GDK_SHIFT_MASK)
-			zoom_to(ggv, ggv->zoom_magstep-1);
+			zoom_to(ggv, ggv_compute_spec(GTK_GS(ggv->gs)->zoom_factor) - 1);
 		else
 			gtk_gs_scroll(GTK_GS(ggv->gs), 0, 64);
 		break;
@@ -288,6 +290,7 @@ void button_release_callback(GtkWidget *widget, GdkEventButton *event,
                         ggv->pan = FALSE;
                         gdk_pointer_ungrab(GDK_CURRENT_TIME);
                         gtk_grab_remove(widget);
+                        gtk_gs_end_scroll(GTK_GS(ggv->gs));
                 }
                 break;
         case 2:
@@ -468,70 +471,22 @@ void show_menubar_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
 
-        ggv->show_menus = GTK_CHECK_MENU_ITEM(widget)->active;
-
-        if (ggv->show_menus) {
-                if((GNOME_APP(ggv->main_window))->menubar == NULL) {
-                        create_menus(ggv, ggv->menudata);
-                        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(ggv->menus_vis),
-                                                        ggv->show_menus);
-                        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(ggv->panel_vis),
-                                                        ggv->show_panel);
-                }
-                gtk_widget_show ((GNOME_APP(ggv->main_window))->menubar->parent);
-        } else {
-                gtk_widget_hide ((GNOME_APP(ggv->main_window))->menubar->parent);
-		gtk_widget_queue_resize (ggv->main_window);
-        }
-        if(ggv->menus_vis &&
-           GTK_CHECK_MENU_ITEM(ggv->menus_vis)->active != ggv->show_menus)
-                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ggv->menus_vis),
-                                               ggv->show_menus);
+        ggv_menus_set_visible(ggv, GTK_CHECK_MENU_ITEM(widget)->active);
 }
 
 void show_toolbar_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
         
-	GnomeDockItem* dock_item = gnome_app_get_dock_item_by_name(GNOME_APP(ggv->main_window), 
-				GNOME_APP_TOOLBAR_NAME);
-
-	ggv->show_toolbar = GTK_CHECK_MENU_ITEM(widget)->active;
-	
-	
-	g_assert(dock_item != NULL);
-
-	if (ggv->show_toolbar) {
-                gtk_widget_show (GTK_WIDGET(dock_item));
-        } else {
-                gtk_widget_hide (GTK_WIDGET(dock_item));
-		gtk_widget_queue_resize (ggv->main_window);
-        }
-        if(ggv->toolbar_vis &&
-           GTK_CHECK_MENU_ITEM(ggv->toolbar_vis)->active != ggv->show_toolbar)
-                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ggv->toolbar_vis),
-                                               ggv->show_toolbar);
+	ggv_toolbar_set_visible(ggv, GTK_CHECK_MENU_ITEM(widget)->active);
 }
 
 
-void hide_panel_callback(GtkWidget *widget, gpointer data)
+void show_panel_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
 
-        ggv->show_panel = GTK_CHECK_MENU_ITEM(widget)->active;
-
-        if (ggv->show_panel) {
-                if(ggv->scrollpane == NULL)
-                        create_sidebar(ggv);
-                gtk_widget_show (ggv->sidebar);
-        } else {
-                gtk_widget_hide (ggv->sidebar);
-		gtk_widget_queue_resize (ggv->main_window);
-        }
-        if(ggv->panel_vis &&
-           GTK_CHECK_MENU_ITEM(ggv->panel_vis)->active != ggv->show_panel)
-                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ggv->panel_vis),
-                                               ggv->show_panel);
+        ggv_panel_set_visible(ggv, GTK_CHECK_MENU_ITEM(widget)->active);
 }
 
 void motion_callback(GtkWidget *widget, GdkEventMotion *event,
@@ -730,7 +685,7 @@ void open_callback(GtkWidget *widget, gpointer data)
 	    ((dirname = g_dirname(GTK_GS (ggv->gs)->gs_filename)) != NULL) &&
             (strcmp(dirname, ".") != 0)) {
                 gtk_file_selection_complete(GTK_FILE_SELECTION(ggv->file_sel),
-                                                    dirname);
+                                            dirname);
 		g_free (dirname);
 	}
         gtk_widget_show(ggv->file_sel);
@@ -753,7 +708,7 @@ void print_callback(GtkWidget *widget, gpointer data)
 		 */
  		gchar *filename;
                 filename = ggv_quote_filename (GTK_GS_GET_PS_FILE(gs));
- 		print_command = g_strdup_printf(gs_print_cmd, filename);
+ 		print_command = g_strdup_printf(ggv_print_cmd, filename);
                 g_free (filename);
 
                 if(print_command) {
@@ -800,7 +755,7 @@ void print_marked_pages_callback(GtkWidget *widget, gpointer data)
 
                 save_marked_pages(ggv, tmp_name);
 
-                print_command = g_strdup_printf(gs_print_cmd, tmp_name);
+                print_command = g_strdup_printf(ggv_print_cmd, tmp_name);
                 if(print_command) {
                         gnome_execute_shell(NULL, print_command);
                         /* FIXME: file tmp_name should be unlinked after
@@ -959,6 +914,28 @@ void scrollpane_right_click_callback(GtkWidget *widget, gpointer data)
         }
 }
 
+gboolean scrollpane_button_press_callback(GtkWidget *widget, GdkEventButton *e,
+                                          gpointer data)
+{
+        ggv_window *ggv = (ggv_window *) data;
+
+        if(e->button == 1)
+                gtk_gs_start_scroll(GTK_GS(ggv->gs));
+
+        return FALSE;
+}
+
+gboolean scrollpane_button_release_callback(GtkWidget *widget, GdkEventButton *e,
+                                            gpointer data)
+{
+        ggv_window *ggv = (ggv_window *) data;
+
+        if(e->button == 1)
+                gtk_gs_end_scroll(GTK_GS(ggv->gs));
+
+        return FALSE;
+}
+
 void
 page_toggled_callback (GtkWidget *widget, gint row, gboolean toggled, gpointer data)
 {
@@ -1037,15 +1014,11 @@ void zoom_callback(GtkWidget *widget, gpointer data)
         ggv_window *ggv = wwd->ggv;
         int new_mag = GPOINTER_TO_INT(wwd->data);
 
-
         if (!GTK_CHECK_MENU_ITEM(widget)->active){
                 /* This is a deactivate signal */
                 return;
         }
 
-        if (ggv->zoom_magstep == new_mag) {
-                return;
-        }
         zoom_to(ggv, new_mag);
 }
 
@@ -1053,15 +1026,8 @@ void zoomin_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *)data;
 
-        /* if there is an option in the zoom menu, we set it,
-           that will zoom to the corresponding zoom mag. step
-           as a side effect. */
-
-        /* If it does not exist (the function returned FALSE)
-           then do the call explicitly */
- 
-        if (!set_corresponding_zoom_menu_option(ggv, ggv->zoom_magstep+1))
-                zoom_to(ggv, ggv->zoom_magstep+1);
+        if (!set_corresponding_zoom_menu_option(ggv, ggv_compute_spec(GTK_GS(ggv->gs)->zoom_factor) + 1))
+                zoom_to(ggv, ggv_compute_spec(GTK_GS(ggv->gs)->zoom_factor) + 1);
 }
 
 
@@ -1069,16 +1035,8 @@ void zoomout_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *)data;
 
-        /* if there is an option in the zoom menu, we set it,
-           that will zoom to the corresponding zoom mag. step
-           as a side effect. */
-
-        /* If it does not exist (the function returned FALSE)
-           then do the call explicitly */
-
-
-        if (!set_corresponding_zoom_menu_option(ggv, ggv->zoom_magstep-1))
-                zoom_to(ggv, ggv->zoom_magstep-1);
+        if (!set_corresponding_zoom_menu_option(ggv, ggv_compute_spec(GTK_GS(ggv->gs)->zoom_factor) - 1))
+                zoom_to(ggv, ggv_compute_spec(GTK_GS(ggv->gs)->zoom_factor) - 1);
 }
 
 void prefs_destroy_callback(GtkWidget *w, ggv_prefs *p)
@@ -1095,14 +1053,18 @@ void prefs_apply_callback(GtkWidget *w, gint page, ggv_prefs *p)
 
         if( page != -1 ) return;
 
-        gtk_gs_defaults_set_interpreter_cmd(gtk_entry_get_text(GTK_ENTRY(p->gs)));
-        gtk_gs_defaults_set_scan_pdf_cmd(gtk_entry_get_text(GTK_ENTRY(p->scan_pdf)));
-        gtk_gs_defaults_set_ungzip_cmd(gtk_entry_get_text (GTK_ENTRY (p->unzip)));
-        gtk_gs_defaults_set_unbzip2_cmd(gtk_entry_get_text (GTK_ENTRY (p->unbzip2)));
+        gtk_gs_defaults_set_interpreter_cmd
+                (g_strdup(gtk_entry_get_text(GTK_ENTRY(p->gs))));
+        gtk_gs_defaults_set_scan_pdf_cmd
+                (g_strdup(gtk_entry_get_text(GTK_ENTRY(p->scan_pdf))));
+        gtk_gs_defaults_set_ungzip_cmd
+                (g_strdup(gtk_entry_get_text (GTK_ENTRY (p->unzip))));
+        gtk_gs_defaults_set_unbzip2_cmd
+                (g_strdup(gtk_entry_get_text (GTK_ENTRY (p->unbzip2))));
 
-        if(gs_print_cmd)
-                g_free(gs_print_cmd);
-        gs_print_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(p->print)));
+        if(ggv_print_cmd)
+                g_free(ggv_print_cmd);
+        ggv_print_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(p->print)));
 
         active = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(p->media))));
         n = gtk_gs_defaults_get_paper_count();
@@ -1115,8 +1077,7 @@ void prefs_apply_callback(GtkWidget *w, gint page, ggv_prefs *p)
         active = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(p->zoom))));
         for(i = 0; i < MENU_ZOOM_SIZE - 1; i++){
                 if(active == p->zoom_choice[i]) {
-                        gs_default_magnification = ZoomMenuMagnificationSteps[i];
-                        gtk_gs_defaults_set_zoom_factor(compute_zoom(gs_default_magnification));
+                        gtk_gs_defaults_set_zoom_factor(ggv_compute_zoom(ZoomMenuMagnificationSteps[i]));
                         break;
                 }
         }
@@ -1142,14 +1103,19 @@ void prefs_apply_callback(GtkWidget *w, gint page, ggv_prefs *p)
         gtk_gs_defaults_set_antialiased(GTK_TOGGLE_BUTTON(p->aa)->active);
         gtk_gs_defaults_set_override_media(GTK_TOGGLE_BUTTON(p->override_media)->active);
         gtk_gs_defaults_set_override_orientation(GTK_TOGGLE_BUTTON(p->override_orientation)->active);
+        gtk_gs_defaults_set_show_scroll_rect(GTK_TOGGLE_BUTTON(p->show_scroll_rect)->active);
 
-        gs_panel = GTK_TOGGLE_BUTTON(p->tbar)->active;
-        gs_save_geometry = GTK_TOGGLE_BUTTON(p->savegeo)->active;
-	gs_toolbar = GTK_TOGGLE_BUTTON(p->toolbar)->active;
-        gs_menubar = GTK_TOGGLE_BUTTON(p->mbar)->active;
-        gs_auto_jump = GTK_TOGGLE_BUTTON(p->auto_jump)->active;
+        ggv_panel = GTK_TOGGLE_BUTTON(p->tbar)->active;
+        ggv_save_geometry = GTK_TOGGLE_BUTTON(p->savegeo)->active;
+	ggv_toolbar = GTK_TOGGLE_BUTTON(p->toolbar)->active;
+        ggv_menubar = GTK_TOGGLE_BUTTON(p->mbar)->active;
+        ggv_auto_jump = GTK_TOGGLE_BUTTON(p->auto_jump)->active;
 
-        apply_gs_prefs(window_list);
+        gtk_gs_defaults_save();
+
+        ggv_prefs_save();
+
+        ggv_prefs_apply(window_list);
 }
 
 void prefs_help_callback(GtkWidget *w, gint page, ggv_prefs *p)
