@@ -23,8 +23,7 @@ void file_save_delete_callback(GtkWidget *widget, GdkEventAny *e, gpointer data)
 void file_save_ok_callback(GtkWidget *widget, gpointer data);
 void show_menubar_callback(GtkWidget *widget, gpointer data);
 void show_toolbar_callback(GtkWidget *widget, gpointer data);
-
-void hide_panel_callback(GtkWidget *widget, gpointer data);
+void show_panel_callback(GtkWidget *widget, gpointer data);
 void motion_callback(GtkWidget *widget, GdkEventMotion *event, gpointer data);
 void key_pressed_event_callback(GtkWidget *widget, GdkEventKey *event, gpointer data);
 void first_page_callback(GtkWidget *widget, gpointer data);
@@ -41,6 +40,8 @@ void recenter_page_callback(GtkWidget *widget, gpointer data);
 void reload_callback(GtkWidget *widget, gpointer data);
 void scrollpane_middle_click_callback(GtkWidget *widget, gpointer data);
 void scrollpane_right_click_callback(GtkWidget *widget, gpointer data);
+  gboolean scrollpane_button_press_callback(GtkWidget *widget, GdkEventButton *e, gpointer data);
+  gboolean scrollpane_button_release_callback(GtkWidget *widget, GdkEventButton *e, gpointer data);
 void save_callback(GtkWidget *widget, gpointer data);
 void save_as_callback(GtkWidget *widget, gpointer data);
 void select_page_callback(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data);
