@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 
-#include <config.h>
+#include "config.h"
+#include <math.h>
 #include <ctype.h>
 #include <sys/stat.h>
 
@@ -14,7 +15,112 @@
 #include "stock/toggleeven.xpm"
 #include "stock/toggleodd.xpm"
 
-#include <math.h>
+GtkGSPaperSize ggv_paper_sizes[] = {
+        {"BBox",         0,  0},
+        {"Letter",       612,  792,},
+        {"Tabloid",      792, 1224,},
+        {"Ledger",       1224,  792,},
+        {"Legal",        612, 1008,},
+        {"Statement",	 396,  612,},
+        {"Executive",	 540,  720,},
+        {"A3",		 842, 1190,},
+        {"A4",		 595,  842,},
+        {"A5",		 420,  595,},
+        {"B4",		 729, 1032,},
+        {"B5",		 516,  729,},
+        {"Folio",        612,  936,},
+        {"Quarto",       610,  780,},
+        {"10x14",        720, 1008,},
+        {NULL,		   0,    0}
+};
+
+gfloat ggv_unit_factors[] = {
+        1.0,
+        25.4,
+        2.54,
+        72.0
+};
+
+char *ggv_orientation_labels[] = {
+        N_("Portrait"),
+        N_("Landscape"),
+        N_("Upside Down"),
+        N_("Seascape"),
+        NULL,
+};
+
+char *ggv_unit_labels[] = {
+        N_("inch"),
+        N_("mm"),
+        N_("cm"),
+        N_("point"),
+        NULL
+};
+
+GSList *
+ggv_split_string (const gchar *string, const gchar *delimiter)
+{
+	const gchar *ptr = string;
+	int pos = 0, escape = 0;
+	char buffer [BUFSIZ];
+	GSList *list = NULL;
+
+	g_return_val_if_fail (string != NULL, NULL);
+	g_return_val_if_fail (delimiter != NULL, NULL);
+
+	while (*ptr) {
+		char c = *ptr++;
+		const gchar *d;
+		int found = 0;
+
+		if (pos >= BUFSIZ) {
+			g_warning ("string too long, aborting");
+			return list;
+		}
+
+		if (escape) {
+			buffer [pos++] = c;
+			escape = 0;
+			continue;
+		}
+
+		if (c == '\\') {
+			escape = 1;
+			continue;
+		}
+
+		for (d = delimiter; *d; d++) {
+			if (c == *d) {
+				buffer [pos++] = 0;
+				list = g_slist_prepend (list, g_strdup (buffer));
+				pos = 0; found = 1;
+				break;
+			}
+		}
+
+		if (!found)
+			buffer [pos++] = c;
+	}
+
+	buffer [pos++] = 0;
+	list = g_slist_prepend (list, g_strdup (buffer));
+
+	return list;
+}
+
+gint
+ggv_get_index_of_string(gchar *string, gchar **strings)
+{
+        guint idx = 0;
+
+        while(strings[idx] != NULL) {
+                if(strcmp(strings[idx], string) == 0)
+                        return idx;
+                idx++;
+        }
+
+        return -1;
+}
 
 /* Quote filename for system call */
 gchar *ggv_quote_filename (const gchar *str)
@@ -133,9 +239,20 @@ void ggv_stock_init (void)
 	gnome_stock_pixmap_register (STOCK_TOGGLE_EVEN, GNOME_STOCK_PIXMAP_REGULAR, &entries[3]);
 	gnome_stock_pixmap_register (STOCK_TOGGLE_ODD, GNOME_STOCK_PIXMAP_REGULAR, &entries[4]);
 	gnome_stock_pixmap_register (STOCK_CLEAR_ALL, GNOME_STOCK_PIXMAP_REGULAR, &entries[5]);
+
 }
 
 gfloat ggv_compute_zoom(gint zoom_spec)
 {
-	return (gfloat)pow(1.2, zoom_spec); /* The Knuth magstep formula rules */
+	return pow(1.2, zoom_spec); /* The Knuth magstep formula rules */
 }
+
+gint ggv_compute_spec(gfloat zoom)
+{
+        zoom = MAX(0.02, zoom);
+        zoom = MIN(10.0, zoom);
+
+        zoom = log(zoom)/log(1.2);
+        return (gint)rint(zoom);
+}
+
