@@ -124,6 +124,7 @@ The DONE message indicates that ghostscript has finished processing.
 #include "gtkgs.h"
 #include "ggvutils.h"
 #include "ps.h"
+#include "gsdefaults.h"
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -150,25 +151,6 @@ static void catchPipe(int i)
         broken_pipe = True;
 }
 
-static GtkGSPaperSize gtk_gs_paper_sizes[] = {
-        {"BBox",         0,  0},
-        {"Letter",       612,  792,},
-        {"Tabloid",      792, 1224,},
-        {"Ledger",       1224,  792,},
-        {"Legal",        612, 1008,},
-        {"Statement",	 396,  612,},
-        {"Executive",	 540,  720,},
-        {"A3",		 842, 1190,},
-        {"A4",		 595,  842,},
-        {"A5",		 420,  595,},
-        {"B4",		 729, 1032,},
-        {"B5",		 516,  729,},
-        {"Folio",        612,  936,},
-        {"Quarto",       610,  780,},
-        {"10x14",        720, 1008,},
-        {NULL,		   0,    0}
-};
-
 /* Forward declarations */
 static void gtk_gs_init (GtkGS *gs);
 static void gtk_gs_class_init (GtkGSClass *klass);
@@ -180,6 +162,8 @@ static gint gtk_gs_widget_event(GtkWidget *widget, GdkEvent *event, gpointer dat
 static void gtk_gs_value_adjustment_changed (GtkAdjustment *adjustment, gpointer data);
 static void gtk_gs_interpreter_message(GtkGS *gs, gchar *msg, gpointer user_data);
 static void gtk_gs_emit_error_msg (GtkGS *gs, const gchar *msg);
+static void gtk_gs_defaults_changed (GConfClient* client, guint cnxn_id,
+                                     GConfEntry *entry, gpointer user_data);
 static void send_ps(GtkGS *gs, long begin, unsigned int len, gboolean close);
 static void set_up_page (GtkGS *gs);
 static void close_pipe (int p[2]);
@@ -193,9 +177,9 @@ static void stop_interpreter (GtkGS *gs);
 static gint start_interpreter (GtkGS *gs);
 gboolean computeSize(void);
 
-static GtkWidgetClass *parent_class;
+static GtkWidgetClass *parent_class = NULL;
 
-static GtkGSClass *gs_class;
+static GtkGSClass *gs_class = NULL;
 
 static gint gtk_gs_signals[LAST_SIGNAL] = { 0 };
 
@@ -214,6 +198,8 @@ gtk_gs_marshaller(GtkObject *object, GtkSignalFunc func,
 static void
 gtk_gs_init (GtkGS *gs)
 {
+        gint i;
+
         gs->bpixmap     = NULL;
         gs->use_bpixmap = TRUE;
 
@@ -273,6 +259,15 @@ gtk_gs_init (GtkGS *gs)
         gs->antialiased = gtk_gs_defaults_get_antialiased();
         gs->override_media = gtk_gs_defaults_get_override_media();
         gs->respect_eof = gtk_gs_defaults_get_respect_eof();
+        gs->watch_doc = gtk_gs_defaults_get_watch_doc();
+        gs->show_scroll_rect = gtk_gs_defaults_get_show_scroll_rect();
+
+        i = 0;
+        gs->gconf_notify_id = g_new0(guint, 6);
+        gs->gconf_notify_id[i++] = gconf_client_notify_add
+                (gtk_gs_defaults_gconf_client(), "/apps/ggv/gtkgs",
+                 (GConfClientNotifyFunc)gtk_gs_defaults_changed,
+                 gs, NULL, NULL);
 }
 
 static void
@@ -308,6 +303,42 @@ gtk_gs_class_init (GtkGSClass *klass)
 
         /* a default handler for "interpreter_message" signal */
         klass->interpreter_message = gtk_gs_interpreter_message;
+
+        gtk_gs_defaults_load();
+}
+
+static void
+gtk_gs_defaults_changed(GConfClient* client, guint cnxn_id,
+                        GConfEntry *entry, gpointer user_data)
+{
+        GtkGS *gs = GTK_GS(user_data);
+
+        if(!strcmp(entry->key, "/apps/ggv/gtkgs/respect_eof"))
+                gtk_gs_set_respect_eof(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/respect_eof", NULL));
+        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/override_orientation"))
+                gtk_gs_set_override_orientation(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/override_orientation",
+                                                                          NULL));
+        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/orientation"))
+                gtk_gs_set_default_orientation(gs, gconf_client_get_int(client, "/apps/ggv/gtkgs/orientation",
+                                                                        NULL));
+        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/zoom"))
+                gtk_gs_set_zoom(gs, gconf_client_get_float(client, "/apps/ggv/gtkgs/zoom",
+                                                           NULL));
+        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/media"))
+                gtk_gs_set_default_media(gs, gconf_client_get_int(client, "/apps/ggv/gtkgs/media",
+                                                                  NULL));
+        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/antialiasing"))
+                gtk_gs_set_antialiasing(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/antialiasing",
+                                                                  NULL));
+        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/override_media"))
+                gtk_gs_set_override_media(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/override_media",
+                                                                    NULL));
+        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/watch_doc"))
+                gtk_gs_set_watch_file(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/watch_doc",
+                                                                NULL));
+        else if(!strcmp(entry->key, "/apps/ggv/gtkgs/show_scroll_rect"))
+                gtk_gs_set_show_scroll_rect(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/show_scroll_rect",
+                                                                      NULL));
 }
 
 /* free message as it was allocated in output() */
@@ -357,6 +388,7 @@ gtk_gs_cleanup (GtkGS *gs)
  		gtk_timeout_remove (gs->timer_tag);
  		gs->timer_tag = 0;
  	}
+        gs->loaded = FALSE;
         gs->llx = 0;
         gs->lly = 0;
         gs->urx = 0;
@@ -367,7 +399,8 @@ static void
 gtk_gs_destroy (GtkObject *object)
 {
         GtkGS *gs;
-  
+        gint i;
+
         g_return_if_fail (object != NULL);
         g_return_if_fail (GTK_IS_GS (object));
         
@@ -375,6 +408,12 @@ gtk_gs_destroy (GtkObject *object)
 
         gtk_gs_cleanup(gs);
 
+        for(i = 0; gs->gconf_notify_id[i] != 0; i++) {
+                gconf_client_notify_remove(gtk_gs_defaults_gconf_client(),
+                                           gs->gconf_notify_id[i]);
+                gs->gconf_notify_id[i] = 0;
+        }
+
         if (GTK_OBJECT_CLASS (parent_class)->destroy)
                 (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -424,8 +463,17 @@ gtk_gs_munge_adjustments (GtkGS *gs)
 			((gfloat) (gs->widget.allocation.height/2 - y))/gs->height;
 	}
 
+
+        if(gs->scroll_start_x != -1 && gs->scroll_start_y != -1)
+                gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
+                                   gs->scroll_start_x, gs->scroll_start_y,
+                                   gs->scroll_width, gs->scroll_height);
 	/* now we want to update the slider. */
 	gdk_window_move (gs->pstarget, x, y);
+        if(gs->scroll_start_x != -1 && gs->scroll_start_y != -1)
+                gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
+                                   gs->scroll_start_x, gs->scroll_start_y,
+                                   gs->scroll_width, gs->scroll_height);
 	gtk_adjustment_changed (gs->hadj);
 	gtk_adjustment_changed (gs->vadj);
 }
@@ -469,6 +517,8 @@ gtk_gs_realize (GtkWidget *widget)
         /* gdk_window_show (gs->pstarget); */
         gdk_window_clear (gs->pstarget);
         gtk_style_set_background (widget->style, gs->pstarget, GTK_STATE_ACTIVE);
+        gs->psgc = gdk_gc_new(gs->pstarget);
+        gdk_gc_set_function(gs->psgc, GDK_INVERT);
 
         gs->width = 0;
         gs->height = 0;
@@ -594,7 +644,15 @@ gtk_gs_value_adjustment_changed (GtkAdjustment *adjustment, gpointer data)
                 newy = gs->widget.allocation.height/2 -
                         (gs->vadj->value - gs->vadj->lower) * height /
                         (gs->vadj->upper - gs->vadj->lower);
+        if(gs->scroll_start_x != -1 && gs->scroll_start_y != -1)
+                gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
+                                   gs->scroll_start_x, gs->scroll_start_y,
+                                   gs->scroll_width, gs->scroll_height);
         gdk_window_move (gs->pstarget, newx, newy);
+        if(gs->scroll_start_x != -1 && gs->scroll_start_y != -1)
+                gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
+                                   gs->scroll_start_x, gs->scroll_start_y,
+                                   gs->scroll_width, gs->scroll_height);
 }
 
 void
@@ -919,6 +977,7 @@ start_interpreter (GtkGS *gs)
 	int std_err[2]; /* pipe from interp stderr */
 #define NUM_ARGS 100
         char *argv[NUM_ARGS];
+        char *filename = NULL;
         int argc = 0;
 
 	if (!gs->gs_filename)
@@ -942,7 +1001,8 @@ start_interpreter (GtkGS *gs)
 
         /* set up the pipes */
         if (gs->send_filename_to_gs) {
-                argv[argc++] = GTK_GS_GET_PS_FILE(gs);
+                filename = g_strconcat("\"", GTK_GS_GET_PS_FILE(gs), "\"", NULL);
+                argv[argc++] = filename;
                 argv[argc++] = "-c";
                 argv[argc++] = "quit";
         }
@@ -960,11 +1020,15 @@ start_interpreter (GtkGS *gs)
         }
 	if (pipe (std_out) == -1) {
 		close_pipe(std_in);
+                if(filename)
+                        g_free(filename);
 		return -1;
 	}
 	if (pipe (std_err) == -1) {
 		close_pipe(std_in);
 		close_pipe(std_out);
+                if(filename)
+                        g_free(filename);
 		return -1;
 	}
 
@@ -975,6 +1039,8 @@ start_interpreter (GtkGS *gs)
 		close_pipe(std_in);
 		close_pipe(std_out);
 		close_pipe(std_err);
+                if(filename)
+                        g_free(filename);
 		return -2;
 		break;
 	case 0: /* child */
@@ -1007,6 +1073,9 @@ start_interpreter (GtkGS *gs)
 
                 execvp(argv[0], argv);
                 
+                if(filename)
+                        g_free(filename);
+
                 /* Notify error */
                 g_print("Unable to execute [%s]\n", argv[0]);
 		_exit(1);
@@ -1261,8 +1330,14 @@ compute_size(GtkGS *gs)
         gs->width = (gint) (new_width * gs->zoom_factor);
         gs->height = (gint) (new_height * gs->zoom_factor);
         if(GTK_WIDGET_REALIZED(gs)) {
-                if(!gdk_window_is_visible(gs->pstarget) && gs->width > 0 && gs->height > 0)
-                        gdk_window_show(gs->pstarget);
+                if(!gs->loaded) {
+                        if(gdk_window_is_visible(gs->pstarget))
+                                gdk_window_hide(gs->pstarget);
+                }
+                else {
+                        if(!gdk_window_is_visible(gs->pstarget) && gs->width > 0 && gs->height > 0)
+                                gdk_window_show(gs->pstarget);
+                }
                 gtk_gs_munge_adjustments (gs);
         }
 
@@ -1368,9 +1443,17 @@ gtk_gs_center_page(GtkGS *gs)
 	g_return_if_fail (gs != NULL);
 	g_return_if_fail (GTK_IS_GS(gs));
 
+        if(gs->scroll_start_x != -1 && gs->scroll_start_y != -1)
+                gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
+                                   gs->scroll_start_x, gs->scroll_start_y,
+                                   gs->scroll_width, gs->scroll_height);
         gdk_window_move (gs->pstarget,
                          (gs->widget.allocation.width - gs->width)/2,
                          (gs->widget.allocation.height - gs->height)/2);
+        if(gs->scroll_start_x != -1 && gs->scroll_start_y != -1)
+                gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
+                                   gs->scroll_start_x, gs->scroll_start_y,
+                                   gs->scroll_width, gs->scroll_height);
         gs->hadj->page_size = ((gfloat) gs->widget.allocation.width)/gs->width;
         gs->vadj->page_size = ((gfloat) gs->widget.allocation.height)/gs->height;
         gs->hadj->value = (gs->hadj->upper - gs->hadj->lower)/2;
@@ -1530,7 +1613,7 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
                                                 timer_callback, gs);
 	}
 
-	return TRUE;
+	return gs->loaded;
 }
 
 
@@ -2096,212 +2179,47 @@ gtk_gs_get_media_index(gchar *string, GtkGSPaperSize *media)
         return -1;
 }
 
-/**
- * defaults for GtkGS widgets
- **/
-
-/* Default values to pass to gtk_gs_init */
-typedef struct _GtkGSDefaults 
-{
-        gboolean   antialiased;
-        gboolean   override_media;
-        gint       default_media;
-	gboolean   watch_doc;
-        gboolean   override_orientation;
-        gboolean   respect_eof;
-        gint       fallback_orientation;
-        gfloat     zoom_factor;
-        gchar      *interpreter_cmd;
-        gchar      *scan_pdf_cmd;
-        gchar      *ungzip_cmd;
-        gchar      *unbzip2_cmd;
-} GtkGSDefaults;
-
-static GtkGSDefaults gtk_gs_defaults = {
-        TRUE, FALSE, 8, TRUE, FALSE, TRUE, 0, 1.0, NULL, NULL, NULL, NULL,
-};
-
 void
-gtk_gs_defaults_set_media(gint iNewPageMedia)
+gtk_gs_start_scroll(GtkGS *gs)
 {
-        gtk_gs_defaults.default_media = iNewPageMedia;
-}
+        gint x, y, w, h;
 
-gint
-gtk_gs_defaults_get_media()
-{
-        return gtk_gs_defaults.default_media;
+        if(!GTK_WIDGET_REALIZED(gs) || !gs->show_scroll_rect)
+                return;
+
+        gdk_window_get_geometry(gs->pstarget, &x, &y, &w, &h, NULL);
+        gs->scroll_start_x = MAX(-x, 0);
+        gs->scroll_start_y = MAX(-y, 0);
+        gs->scroll_width = MIN(gs->widget.allocation.width - 1, w - 1);
+        gs->scroll_height = MIN(gs->widget.allocation.height - 1, h - 1);
+        gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
+                           gs->scroll_start_x, gs->scroll_start_y,
+                           gs->scroll_width, gs->scroll_height);
 }
 
 void
-gtk_gs_defaults_set_watch_doc(gint iNewWatchDoc)
+gtk_gs_end_scroll(GtkGS *gs)
 {
-        gtk_gs_defaults.watch_doc = iNewWatchDoc;
+        if(!GTK_WIDGET_REALIZED(gs) || !gs->show_scroll_rect)
+                return;
+
+        if(gs->scroll_start_x == -1 || gs->scroll_start_y == -1)
+                return;
+
+        gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
+                           gs->scroll_start_x, gs->scroll_start_y,
+                           gs->scroll_width, gs->scroll_height);
+        gs->scroll_start_x = -1;
+        gs->scroll_start_y = -1;
 }
 
-gint
-gtk_gs_defaults_get_watch_doc()
+void gtk_gs_set_show_scroll_rect(GtkGS *gs, gboolean f)
 {
-        return gtk_gs_defaults.watch_doc;
+        gs->show_scroll_rect = f;
 }
 
-void
-gtk_gs_defaults_set_override_media(gboolean bOverMedia)
+gboolean gtk_gs_get_show_scroll_rect(GtkGS *gs)
 {
-        gtk_gs_defaults.override_media = bOverMedia;
+        return gs->show_scroll_rect;
 }
 
-gboolean
-gtk_gs_defaults_get_override_media()
-{
-        return gtk_gs_defaults.override_media;
-}
-
-void
-gtk_gs_defaults_set_override_orientation(gboolean bOverOrien)
-{
-        gtk_gs_defaults.override_orientation = bOverOrien;
-}
-
-gboolean
-gtk_gs_defaults_get_override_orientation()
-{
-        return gtk_gs_defaults.override_orientation;
-}
-
-void
-gtk_gs_defaults_set_antialiased(gint iNewAntialiased)
-{
-        gtk_gs_defaults.antialiased = iNewAntialiased;
-}
-
-gboolean
-gtk_gs_defaults_get_antialiased()
-{
-        return gtk_gs_defaults.antialiased;
-}
-
-void
-gtk_gs_defaults_set_zoom_factor(gfloat fZoom)
-{
-        gtk_gs_defaults.zoom_factor = fZoom;
-}
-
-gfloat
-gtk_gs_defaults_get_zoom_factor()
-{
-        return gtk_gs_defaults.zoom_factor;
-}
-
-void
-gtk_gs_defaults_set_orientation(gint iNewOrientation) {
-        g_assert (
-                  (iNewOrientation == GTK_GS_ORIENTATION_PORTRAIT) ||
-                  (iNewOrientation == GTK_GS_ORIENTATION_LANDSCAPE) ||
-                  (iNewOrientation == GTK_GS_ORIENTATION_UPSIDEDOWN) ||
-                  (iNewOrientation == GTK_GS_ORIENTATION_SEASCAPE)
-                  );
-        gtk_gs_defaults.fallback_orientation = iNewOrientation;
-}
-
-gint
-gtk_gs_defaults_get_orientation()
-{
-        return gtk_gs_defaults.fallback_orientation;
-}
-
-void
-gtk_gs_defaults_set_respect_eof(gboolean resp)
-{
-        gtk_gs_defaults.respect_eof = resp;
-}
-
-gboolean
-gtk_gs_defaults_get_respect_eof()
-{
-        return gtk_gs_defaults.respect_eof;
-}
-
-GtkGSPaperSize *
-gtk_gs_defaults_get_paper_sizes()
-{
-        return gtk_gs_paper_sizes;
-}
-
-gint
-gtk_gs_defaults_get_paper_count()
-{
-        gint n = 0;
-
-        while(gtk_gs_paper_sizes[n].name != NULL)
-                n++;
-
-        return n;
-}
-
-const gchar *
-gtk_gs_defaults_get_interpreter_cmd()
-{
-        if(!gtk_gs_defaults.interpreter_cmd)
-                return GS_PATH;
-        return gtk_gs_defaults.interpreter_cmd;
-}
-
-const gchar *
-gtk_gs_defaults_get_scan_pdf_cmd()
-{
-        if(!gtk_gs_defaults.scan_pdf_cmd)
-                return GS_PATH
-                        " -dNODISPLAY -dQUIET"
-                        " -sPDFname=\"%s\" -sDSCname=\"%s\""
-                        " pdf2dsc.ps -c quit";
-        return gtk_gs_defaults.scan_pdf_cmd;
-}
-
-const gchar *
-gtk_gs_defaults_get_ungzip_cmd()
-{
-        if(!gtk_gs_defaults.ungzip_cmd)
-                return "gzip -cd";
-        return gtk_gs_defaults.ungzip_cmd;
-}
-
-const gchar *
-gtk_gs_defaults_get_unbzip2_cmd()
-{
-        if(!gtk_gs_defaults.unbzip2_cmd)
-                return "bzip2 -cd";
-        return gtk_gs_defaults.unbzip2_cmd;
-}
-
-void
-gtk_gs_defaults_set_interpreter_cmd(const gchar *cmd)
-{
-        if(gtk_gs_defaults.interpreter_cmd)
-                g_free(gtk_gs_defaults.interpreter_cmd);
-        gtk_gs_defaults.interpreter_cmd = g_strdup(cmd);
-}
-
-void
-gtk_gs_defaults_set_scan_pdf_cmd(const gchar *cmd)
-{
-        if(gtk_gs_defaults.scan_pdf_cmd)
-                g_free(gtk_gs_defaults.scan_pdf_cmd);
-        gtk_gs_defaults.scan_pdf_cmd = g_strdup(cmd);
-}
-
-void
-gtk_gs_defaults_set_ungzip_cmd(const gchar *cmd)
-{
-        if(gtk_gs_defaults.ungzip_cmd)
-                g_free(gtk_gs_defaults.ungzip_cmd);
-        gtk_gs_defaults.ungzip_cmd = g_strdup(cmd);
-}
-
-void
-gtk_gs_defaults_set_unbzip2_cmd(const gchar *cmd)
-{
-        if(gtk_gs_defaults.unbzip2_cmd)
-                g_free(gtk_gs_defaults.unbzip2_cmd);
-        gtk_gs_defaults.unbzip2_cmd = g_strdup(cmd);
-}
