@@ -10,6 +10,7 @@
 
 #include <gdk/gdk.h>
 #include <gtk/gtkwidget.h>
+#include <gconf/gconf-client.h>
 #include <errno.h>
 #include <signal.h>
 
@@ -42,6 +43,10 @@ struct _GtkGS
 	GdkWindow *pstarget;                  /* the window passed to gv
                                               * it is a child of widget...
                                               */
+        GdkGC *psgc;
+        gint scroll_start_x, scroll_start_y;
+        gint scroll_width, scroll_height;
+        gboolean show_scroll_rect;
 
 	GdkPixmap *bpixmap;			/* Backing pixmap */
 	int        use_bpixmap;
@@ -105,6 +110,8 @@ struct _GtkGS
         gint        fallback_orientation;  /* Orientation to use if override */
         gint        real_orientation;      /* Real orientation from the document */
         gint       *pages_marked;
+
+        guint      *gconf_notify_id;
 };
 
 struct _GtkGSClass
@@ -117,6 +124,8 @@ struct _GtkGSClass
 	GdkAtom        done_atom;
 	GdkAtom        string_atom;
 
+        GConfClient *gconf_client;
+
         void (*interpreter_message)(GtkGS *, gchar *, gpointer);
 };
 
@@ -179,39 +188,17 @@ const gchar *gtk_gs_get_document_title(GtkGS *widget);
 guint        gtk_gs_get_document_numpages(GtkGS *widget);
 const gchar *gtk_gs_get_document_page_label(GtkGS *widget, int page);
 gint         gtk_gs_count_marked_pages(GtkGS *widget);
+void         gtk_gs_set_show_scroll_rect(GtkGS *gs, gboolean f);
+gboolean     gtk_gs_get_show_scroll_rect(GtkGS *gs);
 
-/* defaults accessors */
-void     gtk_gs_defaults_set_media(gint iNewPageMedia); 
-void     gtk_gs_defaults_set_override_media(gboolean bOverMedia);
-void     gtk_gs_defaults_set_antialiased(gint iNewAntialiased);
-void     gtk_gs_defaults_set_orientation(gint);
-void     gtk_gs_defaults_set_override_orientation(gboolean bOverOrien);
-void     gtk_gs_defaults_set_watch_doc(gint iNewWatchDoc);
-void     gtk_gs_defaults_set_zoom_factor(gfloat fZoom);
-gint     gtk_gs_defaults_get_media(void);
-gboolean gtk_gs_defaults_get_override_media(void);
-gboolean gtk_gs_defaults_get_antialiased(void);
-gint     gtk_gs_defaults_get_orientation(void);
-gboolean gtk_gs_defaults_get_override_orientation(void);
-gfloat   gtk_gs_defaults_get_zoom_factor(void);
-gint     gtk_gs_defaults_get_watch_doc(void) ;
-gint     gtk_gs_defaults_get_paper_count(void);
-gboolean gtk_gs_defaults_get_respect_eof(void);
-void     gtk_gs_defaults_set_respect_eof(gboolean resp);
-GtkGSPaperSize *gtk_gs_defaults_get_paper_sizes(void);
-gint     gtk_gs_defaults_get_paper_count(void);
-const gchar *gtk_gs_defaults_get_interpreter_cmd(void);
-const gchar *gtk_gs_defaults_get_scan_pdf_cmd(void);
-const gchar *gtk_gs_defaults_get_ungzip_cmd(void);
-const gchar *gtk_gs_defaults_get_unbzip2_cmd(void);
-void gtk_gs_defaults_set_interpreter_cmd(const gchar *);
-void gtk_gs_defaults_set_scan_pdf_cmd(const gchar *);
-void gtk_gs_defaults_set_ungzip_cmd(const gchar *);
-void gtk_gs_defaults_set_unbzip2_cmd(const gchar *);
+void         gtk_gs_start_scroll(GtkGS *gs);
+void         gtk_gs_end_scroll(GtkGS *gs);
 
 /* utility functions */
 gint     gtk_gs_get_media_index(gchar *string, GtkGSPaperSize *media);
 
+GConfClient *gtk_gs_get_gconf_client(void);
+
 #define GTK_GS_IS_COMPRESSED(gs)       (gs->gs_filename_unc != NULL)
 #define GTK_GS_GET_PS_FILE(gs)         (GTK_GS_IS_COMPRESSED(gs) ? \
                                         gs->gs_filename_unc : \
